@@ -71,7 +71,8 @@ if __name__ == '__main__':
     #print(res_plu[0])
 
     #CurtoPrazo = Dessem("/Users/andremarcato/Dropbox/Projeto ReadDessem/DS_CCEE_082020_SEMREDE_RV0D01","dessem.arq")
-    MedioPrazo = Newave("/Users/andremarcato/Dropbox/Projeto ReadDessem/Deck_Newave")
+    #MedioPrazo = Newave("/Users/andremarcato/Dropbox/Projeto ReadDessem/Deck_Newave")
+    MedioPrazo = Newave('/Users/andremarcato/Downloads/NewaveJan2021')
 
     uhe = MedioPrazo.confhd.get(287)
 
